@@ -3,7 +3,7 @@ use {
     crate::{
         app::{AppContext, LineNumber},
         command::{ScrollCommand, move_sel},
-        display::{CropWriter, Screen, SPACE_FILLING, W},
+        display::{Screen, W},
         errors::*,
         pattern::{InputPattern, NameMatch},
         skin::PanelSkin,
@@ -23,7 +23,7 @@ use {
         str,
     },
     syntect::highlighting::Style,
-    termimad::Area,
+    termimad::{Area, CropWriter, SPACE_FILLING},
 };
 
 /// a homogeneously colored piece of a line
@@ -267,7 +267,15 @@ impl SyntacticView {
         let old_scroll = self.scroll;
         self.scroll = cmd.apply(self.scroll, self.lines.len(), self.page_height);
         if let Some(idx) = self.selection_idx {
-            if self.scroll != old_scroll && idx >= old_scroll && idx < old_scroll + self.page_height {
+            if self.scroll == old_scroll {
+                let old_selection = self.selection_idx;
+                if cmd.is_up() {
+                    self.selection_idx = Some(0);
+                } else {
+                    self.selection_idx = Some(self.lines.len() - 1);
+                }
+                return self.selection_idx == old_selection;
+            } else  if idx >= old_scroll && idx < old_scroll + self.page_height {
                 if idx + self.scroll < old_scroll {
                     self.selection_idx = Some(0);
                 } else if idx + self.scroll - old_scroll >= self.lines.len() {
@@ -306,7 +314,7 @@ impl SyntacticView {
             .unwrap_or(Color::AnsiValue(240));
         let match_bg = styles.preview_match.get_bg().unwrap_or(Color::AnsiValue(28));
         let code_width = area.width as usize - 1; // 1 char left for scrollbar
-        let scrollbar = area.scrollbar(self.scroll as i32, self.lines.len() as i32);
+        let scrollbar = area.scrollbar(self.scroll, self.lines.len());
         let scrollbar_fg = styles.scrollbar_thumb.get_fg()
             .or_else(|| styles.preview.get_fg())
             .unwrap_or(Color::White);
